@@ -1,5 +1,5 @@
 /* info.c -- Display nodes of Info files in multiple windows.
-   $Id: info.c,v 1.6 2004/02/24 03:25:46 dirt Exp $
+   $Id: info.c,v 1.11 2004/04/11 17:56:45 karl Exp $
 
    Copyright (C) 1993, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
    2004 Free Software Foundation, Inc.
@@ -135,9 +135,8 @@ static char *short_options = "d:n:f:ho:ORws";
 int info_windows_initialized_p = 0;
 
 /* Some "forward" declarations. */
-static void info_short_help ();
-static void init_messages ();
-extern void add_file_directory_to_path ();
+static void info_short_help (void);
+static void init_messages (void);
 
 
 /* **************************************************************** */
@@ -147,9 +146,7 @@ extern void add_file_directory_to_path ();
 /* **************************************************************** */
 
 int
-main (argc, argv)
-     int argc;
-     char **argv;
+main (int argc, char **argv)
 {
   int getopt_long_index;        /* Index returned by getopt_long (). */
   NODE *initial_node;           /* First node loaded by Info. */
@@ -159,9 +156,11 @@ main (argc, argv)
   setlocale (LC_ALL, "");
 #endif
 
+#ifdef ENABLE_NLS
   /* Set the text message domain.  */
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
+#endif
 
   init_messages ();
 
@@ -365,10 +364,10 @@ For more information about these matters, see the files named COPYING.\n"));
   if (!initial_node)
     {
       if (info_recent_file_error)
-        info_error (info_recent_file_error);
+        info_error (info_recent_file_error, NULL, NULL);
       else
-        info_error (msg_cant_find_node,
-                    user_nodenames ? user_nodenames[0] : "Top");
+        info_error ((char *) msg_cant_find_node,
+                    user_nodenames ? user_nodenames[0] : "Top", NULL);
       xexit (1);
     }
 
@@ -396,9 +395,11 @@ For more information about these matters, see the files named COPYING.\n"));
      file name is either "dir", or the contents of user_filename if one
      was specified. */
   {
-    char *errstr, *errarg1, *errarg2;
+    const char *errstr;
+    char *errarg1, *errarg2;
+
     NODE *new_initial_node = info_follow_menus (initial_node, argv + optind,
-                                                &errstr, &errarg1, &errarg2);
+        &errstr, &errarg1, &errarg2);
 
     if (new_initial_node && new_initial_node != initial_node)
       initial_node = new_initial_node;
@@ -424,14 +425,14 @@ For more information about these matters, see the files named COPYING.\n"));
           dump_node_to_file (initial_node, user_output_filename,
                              dump_subnodes);
         else
-          info_error (errstr, errarg1, errarg2);
+          info_error ((char *) errstr, errarg1, errarg2);
       }
     else
       {
 
         if (errstr)
-          begin_info_session_with_error (initial_node, errstr,
-                                         errarg1, errarg2);
+          begin_info_session_with_error (initial_node, (char *) errstr,
+              errarg1, errarg2);
         /* If the user specified `--index-search=STRING' or
            --show-options, start the info session in the node
            corresponding to what they want. */
@@ -513,8 +514,7 @@ For more information about these matters, see the files named COPYING.\n"));
 }
 
 void
-add_file_directory_to_path (filename)
-     char *filename;
+add_file_directory_to_path (char *filename)
 {
   char *directory_name = xstrdup (filename);
   char *temp = filename_non_directory (directory_name);
@@ -548,9 +548,7 @@ int info_error_rings_bell_p = 1;
    then the message is printed in the echo area.  Otherwise, a message is
    output to stderr. */
 void
-info_error (format, arg1, arg2)
-     char *format;
-     void *arg1, *arg2;
+info_error (char *format, void *arg1, void *arg2)
 {
   info_error_was_printed = 1;
 
@@ -586,7 +584,7 @@ info_error (format, arg1, arg2)
 
 /* Produce a scaled down description of the available options to Info. */
 static void
-info_short_help ()
+info_short_help (void)
 {
 #ifdef __MSDOS__
   static const char speech_friendly_string[] = N_("\
@@ -666,7 +664,7 @@ const char *msg_win_too_small;
 const char *msg_cant_make_help;
 
 static void
-init_messages ()
+init_messages (void)
 {
   msg_cant_find_node   = _("Cannot find node `%s'.");
   msg_cant_file_node   = _("Cannot find node `(%s)%s'.");
